@@ -5,6 +5,7 @@ import struct
 from isomedia.exceptions import AtomSpecificationError
 
 CONTAINER_ATOMS = [
+    '----',
     '\xa9alb',
     '\xa9art',
     '\xa9cmt',
@@ -46,10 +47,10 @@ CONTAINER_ATOMS = [
     'schi',
     'sinf',
     'srpp',
+    'stbl',
     'stik',
     'strd',
     'strk',
-    'stbl',
     'tmpo',
     'traf',
     'trak',
@@ -104,13 +105,16 @@ def write_atom_header(atom):
 
     return header_bytes
 
-def interpret_atom(atom_body, definition):
+def interpret_atom(atom_header, atom_body, definition):
     def cast_field(data, size, cast):
         if cast == int:
             data = interpret_int(data, 0, size)
         return data
 
     result = {}
+    atom_body_length = atom_header.size - atom_header.header_length
+
+    bytes_read = 0
 
     for field in definition:
         field_name, field_type = field
@@ -121,11 +125,11 @@ def interpret_atom(atom_body, definition):
             item_count = field_type[2]
 
             data = []
-            while item_count is None or len(data) < item_count:
+            while (item_count is None and bytes_read < atom_body_length) or len(data) < item_count:
                 item_data = atom_body.read(item_length)
-                if not item_data:
-                    break
-                elif len(item_data) != item_length:
+                bytes_read += len(item_data)
+
+                if len(item_data) != item_length:
                     raise AtomSpecificationError
 
                 item_data = cast_field(item_data, item_length, item_cast)
@@ -133,6 +137,8 @@ def interpret_atom(atom_body, definition):
 
         else:
             data = atom_body.read(length)
+            bytes_read += len(data)
+
             if len(data) != length:
                 raise AtomSpecificationError
 
@@ -220,56 +226,56 @@ class Atom(object):
 
 class FullAtom(Atom):
     def __init__(self, atom_header, atom_body, document, parent_atom, file_offset):
-        Atom.__init__(self, atom_header, atom_body, document, parent_atom, file_offset)
+        super(FullAtom, self).__init__(atom_header, atom_body, document, parent_atom, file_offset)
 
         definition = [
             ('version', (1, int)),
             ('flags', (3, None))
         ]
 
-        self.properties.update(interpret_atom(atom_body, definition))
+        self.properties.update(interpret_atom(atom_header, atom_body, definition))
         self._definition['FullAtom'] = definition
 
     def to_bytes(self):
-        written = Atom.to_bytes(self)
+        written = super(FullAtom, self).to_bytes()
         return ''.join([written, write_atom(self.properties, self._definition['FullAtom'])])
 
-class ContainerAtom(Atom):
+class ContainerMixin(object):
     def __init__(self, atom_header, atom_body, document, parent_atom, file_offset):
-        Atom.__init__(self, atom_header, atom_body, document, parent_atom, file_offset)
+        super(ContainerMixin, self).__init__(atom_header, atom_body, document, parent_atom, file_offset)
         self.children = []
 
+    def to_bytes(self):
+        written = super(ContainerMixin, self).to_bytes()
+        return ''.join(chain([written], (child.to_bytes() for child in self.children)))
+
+class ContainerAtom(ContainerMixin, Atom):
+    def __init__(self, atom_header, atom_body, document, parent_atom, file_offset):
+        super(ContainerAtom, self).__init__(atom_header, atom_body, document, parent_atom, file_offset)
+
     def __repr__(self):
         return str({
             'type': self.type,
             'children': self.children
         })
 
-    def to_bytes(self):
-        written = Atom.to_bytes(self)
-        return ''.join(chain([written], (child.to_bytes() for child in self.children)))
-
-    def append_child(self, child):
-        self.size += child.size
-        self.children.append(child)
-
 class GenericAtom(Atom):
     def __init__(self, atom_header, atom_body, document, parent_atom, file_offset):
-        Atom.__init__(self, atom_header, atom_body, document, parent_atom, file_offset)
+        super(GenericAtom, self).__init__(atom_header, atom_body, document, parent_atom, file_offset)
         self._data = atom_body.read()
 
     def get_data(self):
         return self._data
 
     def to_bytes(self):
-        written = Atom.to_bytes(self)
+        written = super(GenericAtom, self).to_bytes()
         return ''.join([written, self.get_data()])
 
 class LazyLoadAtom(Atom):
     LOAD_DATA = False
 
     def __init__(self, atom_header, atom_body, document, parent_atom, file_offset):
-        Atom.__init__(self, atom_header, atom_body, document, parent_atom, file_offset)
+        super(LazyLoadAtom, self).__init__(atom_header, atom_body, document, parent_atom, file_offset)
         self._data = None
 
     def get_data(self):
@@ -281,7 +287,7 @@ class LazyLoadAtom(Atom):
         return self._data
 
     def to_bytes(self):
-        written = Atom.to_bytes(self)
+        written = super(LazyLoadAtom, self).to_bytes()
         return ''.join([written, self.get_data()])
 
 def create_atom(atom_type, atom_body):
